@import url('https://fonts.googleapis.com/css2?family=Jersey+20&display=swap');
@import url('https://fonts.googleapis.com/css2?family=Alegreya+Sans+SC:wght@800&display=swap');

@tailwind base;
@tailwind components;
@tailwind utilities;

@layer base {
  :root {
    --background: 0 0% 100%;
    --foreground: 222.2 47.4% 15%;
    --card: 0 0% 100%;
    --card-foreground: 222.2 47.4% 15%;
    --popover: 0 0% 100%;
    --popover-foreground: 222.2 84% 4.9%;
    --primary: 221.2 83.2% 53.3%;
    --primary-foreground: 210 40% 98%;
    --secondary: 210 40% 96.1%;
    --secondary-foreground: 222.2 47.4% 11.2%;
    --muted: 210 40% 96.1%;
    --muted-foreground: 215.4 16.3% 46.9%;
    --accent: 210 40% 96.1%;
    --accent-foreground: 222.2 47.4% 11.2%;
    --destructive: 0 84.2% 60.2%;
    --destructive-foreground: 210 40% 98%;
    --border: 214.3 31.8% 91.4%;
    --input: 214.3 31.8% 91.4%;
    --ring: 221.2 83.2% 53.3%;
    --radius: 0.5rem;
    --feed-row-default: 0 0% 100%;
    --feed-row-alternate: 210 40% 98%;
  }

  html.dark {
    --background: 222.2 84% 4.9%;
    --foreground: 210 40% 98%;
    --card: 222.2 84% 4.9%;
    --card-foreground: 210 40% 98%;
    --popover: 222.2 84% 4.9%;
    --popover-foreground: 210 40% 98%;
    --primary: 217.2 91.2% 59.8%;
    --primary-foreground: 222.2 47.4% 11.2%;
    --secondary: 217.2 32.6% 17.5%;
    --secondary-foreground: 210 40% 98%;
    --muted: 217.2 32.6% 17.5%;
    --muted-foreground: 215 20.2% 65.1%;
    --accent: 217.2 32.6% 17.5%;
    --accent-foreground: 210 40% 98%;
    --destructive: 0 62.8% 30.6%;
    --destructive-foreground: 210 40% 98%;
    --border: 217.2 32.6% 17.5%;
    --input: 217.2 32.6% 17.5%;
    --ring: 224.3 76.3% 48%;
    --feed-row-default: 222.2 84% 4.9%;
    --feed-row-alternate: 217.2 32.6% 17.5%;
  }

  /* Define fill-background for SVG logo */
  svg path[fill="fill-background"] {
    fill: hsl(var(--background));
  }
}

@layer base {
  * {
    @apply border-border;
  }
  body {
    @apply bg-background text-foreground;
    font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', 'Roboto', 'Oxygen',
      'Ubuntu', 'Cantarell', 'Fira Sans', 'Droid Sans', 'Helvetica Neue',
      sans-serif;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
  }
}

#root {
  height: 100vh;
  width: 100vw;
  overflow: hidden;
}

/* Auto-hiding scrollbars */
@layer utilities {
  /* For Webkit browsers (Chrome, Safari, Edge) */
  .overflow-y-auto::-webkit-scrollbar {
    width: 8px;
  }

  .overflow-y-auto::-webkit-scrollbar-track {
    background: transparent;
  }

  .overflow-y-auto::-webkit-scrollbar-thumb {
    background: transparent;
    border-radius: 4px;
  }

  .overflow-y-auto:hover::-webkit-scrollbar-thumb {
    background: hsl(var(--muted-foreground) / 0.3);
  }

  .overflow-y-auto::-webkit-scrollbar-thumb:hover {
    background: hsl(var(--muted-foreground) / 0.5);
  }

  /* For Firefox */
  .overflow-y-auto {
    scrollbar-width: thin;
    scrollbar-color: transparent transparent;
  }

  .overflow-y-auto:hover {
    scrollbar-color: hsl(var(--muted-foreground) / 0.3) transparent;
  }
}

/* Monaco Editor - Highlighted line for jump to source */
.highlighted-line {
  background-color: rgba(251, 191, 36, 0.2) !important;
  animation: highlight-fade 3s ease-out;
}

html.dark .highlighted-line {
  background-color: rgba(251, 191, 36, 0.15) !important;
}

/* Persistent highlight for jump to source */
.highlighted-line-persistent {
  background-color: rgba(251, 191, 36, 0.1) !important;
  border-left: 3px solid #fbbf24 !important;
}

html.dark .highlighted-line-persistent {
  background-color: rgba(251, 191, 36, 0.08) !important;
  border-left: 3px solid #fbbf24 !important;
}

.highlighted-line-glyph {
  background-color: #fbbf24 !important;
  width: 4px !important;
  margin-left: 3px;
}

@keyframes highlight-fade {
  0% {
    background-color: rgba(251, 191, 36, 0.4);
  }
  100% {
    background-color: rgba(251, 191, 36, 0.2);
  }
}









